@@ -1,6 +1,7 @@
 use std::cmp::min;
 use std::collections::VecDeque;
 use std::io::IoSlice;
+use std::iter::{ExactSizeIterator, Iterator};
 use std::mem::{transmute, MaybeUninit};
 use std::num::NonZeroUsize;
 use std::slice::from_raw_parts_mut;
@@ -51,6 +52,26 @@ impl MpScBytesQueue {
         self.get_pusher().extend(bytes_array)
     }
 
+    pub fn extend_from_iter(&self, iter: impl IntoIterator<Item = Bytes>) {
+        self.get_pusher().extend_from_iter(iter)
+    }
+
+    pub fn extend_from_exact_size_iter<I, It>(&self, iter: I)
+    where
+        I: IntoIterator<Item = Bytes, IntoIter = It>,
+        It: ExactSizeIterator + Iterator<Item = Bytes>,
+    {
+        self.get_pusher().extend_from_exact_size_iter(iter);
+    }
+
+    pub fn reserve(&self, len: usize) {
+        self.get_pusher().reserve(len);
+    }
+
+    pub fn reserve_exact(&self, len: usize) {
+        self.get_pusher().reserve_exact(len);
+    }
+
     /// Return all buffers that need to be flushed.
     ///
     /// Return `None` if there isn't any buffer to flush or another
@@ -99,8 +120,31 @@ impl QueuePusher<'_> {
     }
 
     pub fn extend<const N: usize>(&mut self, bytes_array: [Bytes; N]) {
+        self.0.reserve_exact(N);
         self.0.extend(bytes_array);
     }
+
+    pub fn extend_from_iter(&mut self, iter: impl IntoIterator<Item = Bytes>) {
+        self.0.extend(iter);
+    }
+
+    pub fn extend_from_exact_size_iter<I, It>(&mut self, iter: I)
+    where
+        I: IntoIterator<Item = Bytes, IntoIter = It>,
+        It: ExactSizeIterator + Iterator<Item = Bytes>,
+    {
+        let iter = iter.into_iter();
+        self.0.reserve_exact(iter.len());
+        self.0.extend(iter);
+    }
+
+    pub fn reserve(&mut self, len: usize) {
+        self.0.reserve(len);
+    }
+
+    pub fn reserve_exact(&mut self, len: usize) {
+        self.0.reserve_exact(len);
+    }
 }
 
 #[derive(Debug)]
